##      B&P Glossy Web 2.0 style Button 01
##      Version: 1.2  - [UPDATED: 2012-04-26]

#This version is identical to the version 1.0 in all functionality,
#I just replaced all deprecated functions to work in GIMP 2.7 - 2.8
#
# IT IS NOT COMPATIBLE WITH GIMP 2.6
#
#it creates a glossy Web2.0 style button
#Accepted parameters are dimensions, color,
#corner radius, shadow amount, Add optional text - size, font, color,
# create button states, flatten resulting image(s).
#
# Works best with light colors and low saturation.
#
#Note:
#Developed and tested with Python 2.6.
#
# Original plugin released: 2010-Jun-07
# Andrei Roslovtsev
# www.bytes-and-pixels.com
# andrei[AT]bytes-and-pixels.com


# -----------------------------------------------------------
from gimpfu import *

def make_button(width_var,height_var,rounding,
                shadow_amount,main_col,text_flag,
                text_size,button_txt,font,text_col,
                flatten_flag,states_flag):

    if font==None:
        font='Sans'
        
    # define margin width to accomodate shadow
    margin = shadow_amount*2

    if (margin<8):
        margin=8
    
    # set image dimensions
    img_w = width_var+margin*2
    img_h = height_var+margin*2
        
    image = pdb.gimp_image_new(img_w, img_h, 0)
    pdb.gimp_image_set_filename(image, "Up_image")

    # reset colors to B&W
    pdb.gimp_context_set_default_colors()

    layer_group = pdb.gimp_layer_group_new(image)
    pdb.gimp_item_set_name(layer_group, 'Button_Layers')
    pdb.gimp_image_insert_layer(image, layer_group, None, 0)
    
    
    layer1 = pdb.gimp_layer_new(image, img_w, img_h, 1, 'BG', 100, 0)
    layer2 = pdb.gimp_layer_new(image, img_w, img_h, 1, 'Base Col', 100, 0)
    pdb.gimp_image_insert_layer(image, layer1, layer_group, 0)

    pdb.gimp_image_insert_layer(image, layer2 ,layer_group,-1)
    gimp.pdb.gimp_edit_fill(layer1, 3)

    display = pdb.gimp_display_new(image)


    pdb.gimp_image_select_round_rectangle(image, 2, margin, margin, width_var,
                                          height_var, rounding, rounding)


    pdb.gimp_context_set_foreground(main_col)
    
    pdb.gimp_edit_bucket_fill(layer2, 0, 0, 100, 0, 0, 0, 0)

   
    #Save original selection (shape of the button)
    orig_shape = pdb.gimp_selection_save(image)
    
    #get feather amount based on button dimensions
    if width_var-height_var<0:
        feather=width_var-(width_var/10)
    else:
        feather=height_var-(height_var/10)
    #gimp.pdb.gimp_message(feather)
    
    pdb.gimp_selection_feather(image, feather)
    pdb.gimp_edit_clear(layer2)

    layer2_tmp1=pdb.gimp_layer_copy(layer2, 1)
    layer2_tmp2=pdb.gimp_layer_copy(layer2, 1)
    pdb.gimp_image_insert_layer(image, layer2_tmp1,layer_group,0)
    pdb.gimp_image_insert_layer(image, layer2_tmp2,layer_group,0)

    #merge duplicated layers
    pdb.gimp_item_set_visible(layer1, 0)
  
    # - - o r i g i n a l    m e r g e - - not working in GIMP 2.7.3
    #layer2 = pdb.gimp_image_merge_visible_layers(image, 0)

    # - - w o r k a r o u n d - - - - - - - - - - -  - - - - - - #
    layer2b = pdb.gimp_layer_new_from_visible(image, image, "Base Color")
    pdb.gimp_image_insert_layer(image, layer2b,layer_group,0)

    #delete originals
    pdb.gimp_image_remove_layer(image, layer2_tmp1)
    pdb.gimp_image_remove_layer(image, layer2_tmp2)
    pdb.gimp_image_remove_layer(image, layer2)

    pdb.gimp_item_set_visible(layer1, 1)
    # - - - - - - - - - - - - - - - - - - - - - - -  - - - - - - #

    #load original shape selection
    pdb.gimp_image_select_item(image, 2, orig_shape)

#Add matte layer --------------------------------------------------------
    pdb.gimp_context_set_background((255,255,255))
    layer_matte = pdb.gimp_layer_new(image, img_w, img_h, 1, 'Matte', 100, 0)
    pdb.gimp_image_insert_layer(image, layer_matte,layer_group, 1)
    pdb.gimp_image_select_item(image, 2, orig_shape)
    
    pdb.gimp_edit_bucket_fill(layer_matte, 1, 0, 100, 0, 0, 0, 0)
    
#Add reflection ---------------------------------------------------------

    pdb.gimp_image_select_ellipse(image, 3, img_w*-1, img_h/2, img_w*3, (width_var+height_var)/2)

    layer3 = pdb.gimp_layer_new(image, img_w, img_h, 1, 'Reflection', 25, 3)
    pdb.gimp_image_insert_layer(image, layer3,layer_group, 0)

    #pdb.gimp_selection_shrink(image, 10)

    pdb.gimp_context_set_default_colors()
    
    #pdb.gimp_context_set_gradient('FG to Transparent')
    pdb.gimp_edit_blend(layer3, 2, 0, 0, 100, 0, 0, 0, 0,
                        0, 0, 1, img_w/2, img_h,
                        img_w/2, img_h/3)

# Specular 1 layer -----------------------------------------------------------    
    
    layer4 = pdb.gimp_layer_new(image, img_w, img_h, 1, 'Specular', 100, 0)
    pdb.gimp_image_insert_layer(image, layer4,layer_group, 0)

    #load original shape selection
    pdb.gimp_image_select_item(image, 2, orig_shape)

    pdb.gimp_selection_shrink(image, margin*0.2)
    
    pdb.gimp_image_select_rectangle(image, 1,0,margin+margin*0.4,img_w+margin*2,
                                    img_h+margin*2)
    pdb.gimp_context_swap_colors()
    
    pdb.gimp_edit_blend(layer4, 2, 0, 1, 100, 0, 0, 0, 0,
                        0, 0, 1, img_w/2, 0, rounding*2, 0)

    pdb.gimp_context_swap_colors()
    pdb.gimp_selection_none(image)

    pdb.plug_in_gauss(image, layer4, width_var*0.02, height_var*0.01, 0)

# Shadow layer ---------------------------------------------------------------
    if (shadow_amount!=0):
        
        layer5 = pdb.gimp_layer_new(image, img_w, img_h, 1, 'Shadow', 75, 0)
        pdb.gimp_image_insert_layer(image, layer5,layer_group, 3)

        #load original shape selection
        pdb.gimp_image_select_item(image, 2, orig_shape)

        pdb.gimp_selection_border(image,shadow_amount)

        pdb.gimp_edit_bucket_fill(layer5, 0, 0, 100, 0, 0, 0, 0)

        pdb.gimp_selection_none(image)

        # Do blur operation a few times depending on shadow_amount
        i=0
        while (i<shadow_amount):
            # set limit for itererations
            if (shadow_amount<10):
                pdb.plug_in_blur(image, layer5)
            i=i+1

#Reflection 2 layer (window frame)--------------------------------------------
    
    layer6 = pdb.gimp_layer_new(image, img_w, img_h,
                                1, 'Window Refl', 30, 4)
    pdb.gimp_image_insert_layer(image, layer6,layer_group, 0)

    pdb.gimp_selection_all(image)
    pdb.gimp_image_select_rectangle(image, 1,img_w/2-margin/2, 0, margin, img_h)
    pdb.gimp_image_select_rectangle(image, 1, 0, img_h/2-margin/2, img_w, margin)

    pdb.gimp_edit_bucket_fill(layer6, 1, 0, 100, 0, 0, 0, 0)
    pdb.gimp_selection_none(image)
    pdb.gimp_layer_scale(layer6, img_w/3, img_h/3, 0)
    pdb.gimp_layer_translate(layer6, img_w/2-rounding/2, margin*1.2)

    pdb.plug_in_lens_distortion(image, layer6, 100, 80,
                                50, 0, 0, 0)
    pdb.gimp_layer_resize_to_image_size(layer6)

# Text Layer (optional) --------------------------------------------
    if text_flag == 1:
        layer7 = pdb.gimp_text_layer_new(image, button_txt, font, text_size, 0)
        pdb.gimp_image_insert_layer(image, layer7,layer_group, 0)
        pdb.gimp_text_layer_set_color(layer7, text_col)
        
        txt_x = (image.width-layer7.width)/2
        txt_y = (image.height-layer7.height)/2
        layer7.set_offsets(txt_x, txt_y)

# Create state images -----------------------------------------------
    if states_flag == 1:
        over_image = pdb.gimp_image_duplicate(image)
        down_image = pdb.gimp_image_duplicate(image)

        pdb.gimp_image_set_filename(over_image, "Over_image")
        pdb.gimp_image_set_filename(down_image, "Down_image")

        display = pdb.gimp_display_new(over_image)
        display = pdb.gimp_display_new(down_image)

        # Edit "Over" state
        o_layer5 = find_layer_by_name(over_image,"Shadow")
        o_layer6 = find_layer_by_name(over_image,"Window Refl")
        o_layer3 = find_layer_by_name(over_image,"Reflection")

        
        pdb.gimp_layer_set_opacity(o_layer5, 90) #shadow layer
        pdb.gimp_layer_set_opacity(o_layer6, 80) #window refl layer
        pdb.gimp_layer_set_opacity(o_layer3, 50) # refl layer


        # Edit "Down" state
        d_layer4 = find_layer_by_name(down_image,"Base Color")
        pdb.gimp_image_set_active_layer(down_image, d_layer4)
        pdb.gimp_hue_saturation( d_layer4, 0, -50, 0, -50)

# Flatten Image routine ---------------------------------------------
    if flatten_flag == 1:
        pdb.gimp_image_remove_layer(image, layer1)
        layer1 = pdb.gimp_image_merge_visible_layers(image, 0)

        if states_flag == 1:
            pdb.gimp_image_remove_layer(over_image, find_layer_by_name(over_image,"BG"))
            o_layer1 = pdb.gimp_image_merge_visible_layers(over_image, 0)

            pdb.gimp_image_remove_layer(down_image, find_layer_by_name(down_image,"BG"))
            d_layer1 = pdb.gimp_image_merge_visible_layers(down_image, 0)
        
    pdb.gimp_image_set_active_layer(image, layer1)

    
#--------------------------------------------------------------------
#Helper functions
#Author:  Joao S. O. Bueno
#source:  http://www.gimpusers.com/forums/gimp-user/
#    11870-set-active-layer-by-referring-to-its-name

# Edited by Andrei Roslovtsev:
# 2012-04-25
# to address limitation of 2.7+ regarding layer groups:
# see this post:
# http://stackoverflow.com/questions/10221926/how-get-sublayers-from-group-layer-with-python-in-gimp
def find_layer_by_name (image, name):
    for layer in image.layers:
        if pdb.gimp_item_is_group(layer):
            gr = layer
            gr_items = pdb.gimp_item_get_children(layer)
            for index in gr_items[1]:
                item = gimp.Item.from_id(index)
                if item.name == name:
                    return item


        if layer.name == name:
            return layer
            return None


#---------------------------------------------------------------------
register(
    "Glossy_Button_01",
    N_("Create a glossy button"),
    "Create a glossy Web 2.0 style button",
    "Andrei Roslovtsev",
    "www.bytes-and-pixels.com",
    "2010",
    N_("_Glossy button 01"),
    #"RGB*, GRAY*",
    "",
    [
        (PF_SPINNER, "width_var", "Button Width:", 140, (1, 2000, 1)),
        (PF_SPINNER, "height_var", "Button Height:", 50, (1, 2000, 1)),
        (PF_SPINNER, "rounding", "Corner Radius:", 12, (0, 100, 1)),
        (PF_SPINNER, "shadow_amount", "Shadow amount:", 4, (0, 100, 1)),
        (PF_COLOR, "main_col", "Color:", (161, 179, 196) ),
        (PF_TOGGLE, "text_flag", "Add Text:", 1),
        (PF_SPINNER, "text_size", "Text Size (px):", 20, (1, 2000, 1)),
        (PF_STRING, "button_txt", "Text:", "Download"),
        (PF_FONT, "font", "FONT:", 0),
        (PF_COLOR, "text_col", "Text Color:", (0, 0, 0) ),
        (PF_TOGGLE, "flatten_flag", "Flatten Image", 0),
        (PF_TOGGLE, "states_flag", "Generate Button States", 0)
    ],
    [],
    make_button,
    menu="<Toolbox>/Xtns/Web Page Themes/www.bytes-and-pixels.com",
    domain=("gimp20-python", gimp.locale_directory)
    )

main()
